@@ -1,8 +1,8 @@
 from __future__ import unicode_literals
 
-import sys
 import threading
 import time
+import unittest
 
 import pytest
 
@@ -17,12 +17,6 @@ from prometheus_client.exposition import (
     passthrough_redirect_handler,
 )
 
-if sys.version_info < (2, 7):
-    # We need the skip decorators from unittest2 on Python 2.6.
-    import unittest2 as unittest
-else:
-    import unittest
-
 try:
     from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
 except ImportError:
@@ -98,7 +92,6 @@ ss_sum{a="c",b="d"} 17.0
 ss_created{a="c",b="d"} 123.456
 """, generate_latest(self.registry))
 
-    @unittest.skipIf(sys.version_info < (2, 7), "Test requires Python 2.7+.")
     def test_histogram(self):
         s = Histogram('hh', 'A histogram', registry=self.registry)
         s.observe(0.05)
@@ -376,10 +369,6 @@ class TestPushGateway(unittest.TestCase):
         # ensure the redirect took place at the expected redirect location.
         self.assertEqual(self.requests[1][0].path, "/" + self.redirect_flag)
 
-    @unittest.skipIf(
-        sys.platform == "darwin",
-        "instance_ip_grouping_key() does not work on macOS."
-    )
     def test_instance_ip_grouping_key(self):
         self.assertTrue('' != instance_ip_grouping_key()['instance'])
 
